// -*- C++ -*-
/*!
 * @file  Demultiplexer.cpp
 * @brief Demultiplexer component
 * $Date$
 *
 * $Id$
 */

#include "Demultiplexer.h"
#include <iostream>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define BOOL 5


// Module specification
// <rtc-template block="module_spec">
static const char* demultiplexer_spec[] =
  {
    "implementation_id", "Demultiplexer",
    "type_name",         "Demultiplexer",
    "description",       "Demultiplexer component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    "conf.default.OutPort_Num", "3",

    ""
  };
// </rtc-template>

Demultiplexer::Demultiplexer(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_larrinIn("longarrayin", m_larrin),
    m_sarrinIn("shortarrayin", m_sarrin),
    m_farrinIn("floatarrayin", m_farrin),
    m_darrinIn("doublearrayin", m_darrin),
    m_barrinIn("booleanarrayin", m_barrin),

    m_longout0Out("longout0", m_longout0),
    m_longout1Out("longout1", m_longout1),
    m_longout2Out("longout2", m_longout2),
    m_longout3Out("longout3", m_longout3),
    m_longout4Out("longout4", m_longout4),
    m_longout5Out("longout5", m_longout5),
    m_longout6Out("longout6", m_longout6),
    m_longout7Out("longout7", m_longout7),
    m_longout8Out("longout8", m_longout8),
    m_longout9Out("longout9", m_longout9),
    m_shortout0Out("shortout0", m_shortout0),
    m_shortout1Out("shortout1", m_shortout1),
    m_shortout2Out("shortout2", m_shortout2),
    m_shortout3Out("shortout3", m_shortout3),
    m_shortout4Out("shortout4", m_shortout4),
    m_shortout5Out("shortout5", m_shortout5),
    m_shortout6Out("shortout6", m_shortout6),
    m_shortout7Out("shortout7", m_shortout7),
    m_shortout8Out("shortout8", m_shortout8),
    m_shortout9Out("shortout9", m_shortout9),
    m_floatout0Out("floatout0", m_floatout0),
    m_floatout1Out("floatout1", m_floatout1),
    m_floatout2Out("floatout2", m_floatout2),
    m_floatout3Out("floatout3", m_floatout3),
    m_floatout4Out("floatout4", m_floatout4),
    m_floatout5Out("floatout5", m_floatout5),
    m_floatout6Out("floatout6", m_floatout6),
    m_floatout7Out("floatout7", m_floatout7),
    m_floatout8Out("floatout8", m_floatout8),
    m_floatout9Out("floatout9", m_floatout9),
    m_doubleout0Out("doubleout0", m_doubleout0),
    m_doubleout1Out("doubleout1", m_doubleout1),
    m_doubleout2Out("doubleout2", m_doubleout2),
    m_doubleout3Out("doubleout3", m_doubleout3),
    m_doubleout4Out("doubleout4", m_doubleout4),
    m_doubleout5Out("doubleout5", m_doubleout5),
    m_doubleout6Out("doubleout6", m_doubleout6),
    m_doubleout7Out("doubleout7", m_doubleout7),
    m_doubleout8Out("doubleout8", m_doubleout8),
    m_doubleout9Out("doubleout9", m_doubleout9),
    m_boolout0Out("boolout0", m_boolout0),
    m_boolout1Out("boolout1", m_boolout1),
    m_boolout2Out("boolout2", m_boolout2),
    m_boolout3Out("boolout3", m_boolout3),
    m_boolout4Out("boolout4", m_boolout4),
    m_boolout5Out("boolout5", m_boolout5),
    m_boolout6Out("boolout6", m_boolout6),
    m_boolout7Out("boolout7", m_boolout7),
    m_boolout8Out("boolout8", m_boolout8),
    m_boolout9Out("boolout9", m_boolout9),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("datain", m_datainIn);
  
  // Set OutPort buffer
  //registerOutPort("dataout", m_dataoutOut);
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("demultiplexer", "demultiplexer", m_demultiplexer);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

Demultiplexer::~Demultiplexer()
{
}


RTC::ReturnCode_t Demultiplexer::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("OutPort_Num", m_OutPort_Num, "3");
  
  // </rtc-template>
  std::cout << "Demultiplexer Start." << std::endl;  
  return RTC::RTC_OK;
}

RTC::ReturnCode_t Demultiplexer::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}



bool svc_flag=false;
int svc_type=0;
int svc_num=0;
bool activate_flag=true;

RTC::ReturnCode_t Demultiplexer::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){

  if(svc_flag)
    type = svc_type;
  else{
    if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double" && m_DataType!="bool"){
      std::cout << "DataType ERROR" << std::endl;
      return RTC::RTC_ERROR;
    }
    if(m_DataType=="short")
      type = SHORT;
    else if(m_DataType=="long")
      type = LONG;
    else if(m_DataType=="float")
      type = FLOAT;
    else if(m_DataType=="double")
      type = DOUBLE;
    else if(m_DataType=="bool")
      type = BOOL;
  }


  if(svc_flag)
    num = svc_num;
  else
    num = m_OutPort_Num;

  if(1 > num || num > 10){
    std::cout << "OutPort_Num ERROR" << std::endl;
    return RTC::RTC_ERROR;
  }  


  switch(type){
    case SHORT:
      registerInPort("shortArrayin", m_sarrinIn);
      if(num > 0)
        registerOutPort("shortout0", m_shortout0Out);
      if(num > 1)
        registerOutPort("shortout1", m_shortout1Out);
      if(num > 2)
        registerOutPort("shortout2", m_shortout2Out);
      if(num > 3)
        registerOutPort("shortout3", m_shortout3Out);
      if(num > 4)
        registerOutPort("shortout4", m_shortout4Out);
      if(num > 5)
        registerOutPort("shortout5", m_shortout5Out);
      if(num > 6)
        registerOutPort("shortout6", m_shortout6Out);
      if(num > 7)
        registerOutPort("shortout7", m_shortout7Out);
      if(num > 8)
        registerOutPort("shortout8", m_shortout8Out);
      if(num > 9)
        registerOutPort("shortout9", m_shortout9Out);
    break;

    case LONG:
      registerInPort("longArrayin", m_larrinIn);
      if(num > 0)
        registerOutPort("longout0", m_longout0Out);
      if(num > 1)
        registerOutPort("longout1", m_longout1Out);
      if(num > 2)
        registerOutPort("longout2", m_longout2Out);
      if(num > 3)
        registerOutPort("longout3", m_longout3Out);
      if(num > 4)
        registerOutPort("longout4", m_longout4Out);
      if(num > 5)
        registerOutPort("longout5", m_longout5Out);
      if(num > 6)
        registerOutPort("longout6", m_longout6Out);
      if(num > 7)
        registerOutPort("longout7", m_longout7Out);
      if(num > 8)
        registerOutPort("longout8", m_longout8Out);
      if(num > 9)
        registerOutPort("longout9", m_longout9Out);
    break;

    case FLOAT:
      registerInPort("floatArrayin", m_farrinIn);
      if(num > 0)
        registerOutPort("floatout0", m_floatout0Out);
      if(num > 1)
        registerOutPort("floatout1", m_floatout1Out);
      if(num > 2)
        registerOutPort("floatout2", m_floatout2Out);
      if(num > 3)
        registerOutPort("floatout3", m_floatout3Out);
      if(num > 4)
        registerOutPort("floatout4", m_floatout4Out);
      if(num > 5)
        registerOutPort("floatout5", m_floatout5Out);
      if(num > 6)
        registerOutPort("floatout6", m_floatout6Out);
      if(num > 7)
        registerOutPort("floatout7", m_floatout7Out);
      if(num > 8)
        registerOutPort("floatout8", m_floatout8Out);
      if(num > 9)
        registerOutPort("floatout9", m_floatout9Out);
    break;

    case DOUBLE:
      registerInPort("doubleArrayin", m_darrinIn);
      if(num > 0)
        registerOutPort("doubleout0", m_doubleout0Out);
      if(num > 1)
        registerOutPort("doubleout1", m_doubleout1Out);
      if(num > 2)
        registerOutPort("doubleout2", m_doubleout2Out);
      if(num > 3)
        registerOutPort("doubleout3", m_doubleout3Out);
      if(num > 4)
        registerOutPort("doubleout4", m_doubleout4Out);
      if(num > 5)
        registerOutPort("doubleout5", m_doubleout5Out);
      if(num > 6)
        registerOutPort("doubleout6", m_doubleout6Out);
      if(num > 7)
        registerOutPort("doubleout7", m_doubleout7Out);
      if(num > 8)
        registerOutPort("doubleout8", m_doubleout8Out);
      if(num > 9)
        registerOutPort("doubleout9", m_doubleout9Out);
    break;

    case BOOL:
      registerInPort("booleanArrayin", m_barrinIn);
      if(num > 0)
        registerOutPort("booleanout0", m_boolout0Out);
      if(num > 1)
        registerOutPort("booleanout1", m_boolout1Out);
      if(num > 2)
        registerOutPort("booleanout2", m_boolout2Out);
      if(num > 3)
        registerOutPort("booleanout3", m_boolout3Out);
      if(num > 4)
        registerOutPort("booleanout4", m_boolout4Out);
      if(num > 5)
        registerOutPort("booleanout5", m_boolout5Out);
      if(num > 6)
        registerOutPort("booleanout6", m_boolout6Out);
      if(num > 7)
        registerOutPort("booleanout7", m_boolout7Out);
      if(num > 8)
        registerOutPort("booleanout8", m_boolout8Out);
      if(num > 9)
        registerOutPort("booleanout9", m_boolout9Out);
    break;
  }

  activate_flag=false;
  }

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Demultiplexer::onDeactivated(RTC::UniqueId ec_id)
{
  m_sarrinIn.read();
  m_larrinIn.read();
  m_farrinIn.read();
  m_darrinIn.read();
  m_barrinIn.read();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Demultiplexer::onExecute(RTC::UniqueId ec_id)
{
  bool flag=false;
  std::vector<short> sarrnum;
  std::vector<long> larrnum;
  std::vector<float> farrnum;
  std::vector<double> darrnum;
  std::vector<bool> barrnum;

  if(m_sarrinIn.isNew()){
    m_sarrinIn.read();
    if(num != m_sarrin.data.length()){
      std::cout << "data.length ERROR." << std::endl;
      return RTC::RTC_ERROR;
    }
    for(int i=0; i<num; i++)
      sarrnum.push_back(m_sarrin.data[i]);
    flag = true;
  }
  if(m_larrinIn.isNew()){
    m_larrinIn.read();
    if(num != m_larrin.data.length()){
      std::cout << "data.length ERROR." << std::endl;
      return RTC::RTC_ERROR;
    }
    for(int i=0; i<num; i++)
      larrnum.push_back(m_larrin.data[i]);
    flag = true;
  }
  if(m_farrinIn.isNew()){
    m_farrinIn.read();
    if(num != m_farrin.data.length()){
      std::cout << "data.length ERROR." << std::endl;
      return RTC::RTC_ERROR;
    }
    for(int i=0; i<num; i++)
      farrnum.push_back(m_farrin.data[i]);
    flag = true;
  }
  if(m_darrinIn.isNew()){
    m_darrinIn.read();
    if(num != m_darrin.data.length()){
      std::cout << "data.length ERROR." << std::endl;
      return RTC::RTC_ERROR;
    }
    for(int i=0; i<num; i++)
      darrnum.push_back(m_darrin.data[i]);
    flag = true;
  }
  if(m_barrinIn.isNew()){
    m_barrinIn.read();
    if(num != m_barrin.data.length()){
      std::cout << "data.length ERROR." << std::endl;
      return RTC::RTC_ERROR;
    }
    for(int i=0; i<num; i++)
      barrnum.push_back(m_barrin.data[i]);
    flag = true;
  }

  if(flag){
    if(num > 0){
      if(sarrnum.size() > 0)
        m_shortout0.data = sarrnum[0];
      m_shortout0Out.write();
      if(larrnum.size() > 0)
        m_longout0.data = larrnum[0];
      m_longout0Out.write();
      if(farrnum.size() > 0)
        m_floatout0.data = farrnum[0];
      m_floatout0Out.write();
      if(darrnum.size() > 0)
        m_doubleout0.data = darrnum[0];
      m_doubleout0Out.write();
      if(barrnum.size() > 0)
        m_boolout0.data = barrnum[0];
      m_boolout0Out.write();
    }
    if(num > 1){
      if(sarrnum.size() > 1)
        m_shortout1.data = sarrnum[1];
      m_shortout1Out.write();
      if(larrnum.size() > 1)
        m_longout1.data = larrnum[1];
      m_longout1Out.write();
      if(farrnum.size() > 1)
        m_floatout1.data = farrnum[1];
      m_floatout1Out.write();
      if(darrnum.size() > 1)
        m_doubleout1.data = darrnum[1];
      m_doubleout1Out.write();
      if(barrnum.size() > 1)
        m_boolout1.data = barrnum[1];
      m_boolout1Out.write();
    }
    if(num > 2){
      if(sarrnum.size() > 2)
        m_shortout2.data = sarrnum[2];
      m_shortout2Out.write();
      if(larrnum.size() > 2)
        m_longout2.data = larrnum[2];
      m_longout2Out.write();
      if(farrnum.size() > 2)
        m_floatout2.data = farrnum[2];
      m_floatout2Out.write();
      if(darrnum.size() > 2)
        m_doubleout2.data = darrnum[2];
      m_doubleout2Out.write();
      if(barrnum.size() > 2)
        m_boolout2.data = barrnum[2];
      m_boolout2Out.write();
    }
    if(num > 3){
      if(sarrnum.size() > 3)
        m_shortout3.data = sarrnum[3];
      m_shortout3Out.write();
      if(larrnum.size() > 3)
        m_longout3.data = larrnum[3];
      m_longout3Out.write();
      if(farrnum.size() > 3)
        m_floatout3.data = farrnum[3];
      m_floatout3Out.write();
      if(darrnum.size() > 3)
        m_doubleout3.data = darrnum[3];
      m_doubleout3Out.write();
      if(barrnum.size() > 3)
        m_boolout3.data = barrnum[3];
      m_boolout3Out.write();
    }
    if(num > 4){
      if(sarrnum.size() > 4)
        m_shortout4.data = sarrnum[4];
      m_shortout4Out.write();
      if(larrnum.size() > 4)
        m_longout4.data = larrnum[4];
      m_longout4Out.write();
      if(farrnum.size() > 4)
        m_floatout4.data = farrnum[4];
      m_floatout4Out.write();
      if(darrnum.size() > 4)
        m_doubleout4.data = darrnum[4];
      m_doubleout4Out.write();
      if(barrnum.size() > 4)
        m_boolout4.data = barrnum[4];
      m_boolout4Out.write();
    }
    if(num > 5){
      if(sarrnum.size() > 5)
        m_shortout5.data = sarrnum[5];
      m_shortout5Out.write();
      if(larrnum.size() > 5)
        m_longout5.data = larrnum[5];
      m_longout5Out.write();
      if(farrnum.size() > 5)
        m_floatout5.data = farrnum[5];
      m_floatout5Out.write();
      if(darrnum.size() > 5)
        m_doubleout5.data = darrnum[5];
      m_doubleout5Out.write();
      if(barrnum.size() > 5)
        m_boolout5.data = barrnum[5];
      m_boolout5Out.write();
    }
    if(num > 6){
      if(sarrnum.size() > 6)
        m_shortout6.data = sarrnum[6];
      m_shortout6Out.write();
      if(larrnum.size() > 6)
        m_longout6.data = larrnum[6];
      m_longout6Out.write();
      if(farrnum.size() > 6)
        m_floatout6.data = farrnum[6];
      m_floatout6Out.write();
      if(darrnum.size() > 6)
        m_doubleout6.data = darrnum[6];
      m_doubleout6Out.write();
      if(barrnum.size() > 6)
        m_boolout6.data = barrnum[6];
      m_boolout6Out.write();
    }
    if(num > 7){
      if(sarrnum.size() > 7)
        m_shortout7.data = sarrnum[7];
      m_shortout7Out.write();
      if(larrnum.size() > 7)
        m_longout7.data = larrnum[7];
      m_longout7Out.write();
      if(farrnum.size() > 7)
        m_floatout7.data = farrnum[7];
      m_floatout7Out.write();
      if(darrnum.size() > 7)
        m_doubleout7.data = darrnum[7];
      m_doubleout7Out.write();
      if(barrnum.size() > 7)
        m_boolout7.data = barrnum[7];
      m_boolout7Out.write();
    }
    if(num > 8){
      if(sarrnum.size() > 8)
        m_shortout8.data = sarrnum[8];
      m_shortout8Out.write();
      if(larrnum.size() > 8)
        m_longout8.data = larrnum[8];
      m_longout8Out.write();
      if(farrnum.size() > 8)
        m_floatout8.data = farrnum[8];
      m_floatout8Out.write();
      if(darrnum.size() > 8)
        m_doubleout8.data = darrnum[8];
      m_doubleout8Out.write();
      if(barrnum.size() > 8)
        m_boolout8.data = barrnum[8];
      m_boolout8Out.write();
    }
    if(num > 9){
      if(sarrnum.size() > 9)
        m_shortout9.data = sarrnum[9];
      m_shortout9Out.write();
      if(larrnum.size() > 9)
        m_longout9.data = larrnum[9];
      m_longout9Out.write();
      if(farrnum.size() > 9)
        m_floatout9.data = farrnum[9];
      m_floatout9Out.write();
      if(darrnum.size() > 9)
        m_doubleout9.data = darrnum[9];
      m_doubleout9Out.write();
      if(barrnum.size() > 9)
        m_boolout9.data = barrnum[9];
      m_boolout9Out.write();
    }
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Demultiplexer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Demultiplexer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Demultiplexer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DemultiplexerInit(RTC::Manager* manager)
  {
    RTC::Properties profile(demultiplexer_spec);
    manager->registerFactory(profile,
                             RTC::Create<Demultiplexer>,
                             RTC::Delete<Demultiplexer>);
  }
  
};


