// -*- C++ -*-
/*!
 * @file  Duplicator.cpp
 * @brief Duplicator component
 * $Date$
 *
 * $Id$
 */

#include "Duplicator.h"
#include <iostream>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define BOOL 5
#define SHORTARRAY 10
#define LONGARRAY 20
#define FLOATARRAY 30
#define DOUBLEARRAY 40
#define BOOLARRAY 50

// Module specification
// <rtc-template block="module_spec">
static const char* duplicator_spec[] =
  {
    "implementation_id", "Duplicator",
    "type_name",         "Duplicator",
    "description",       "Duplicator component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    "conf.default.OutPort_Num", "2",

    ""
  };
// </rtc-template>

Duplicator::Duplicator(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_longinIn("longin", m_longin),
    m_shortinIn("shortin", m_shortin),
    m_floatinIn("floatin", m_floatin),
    m_doubleinIn("doublein", m_doublein),
    m_boolinIn("boolin", m_boolin),

    m_larrinIn("longarrayin", m_larrin),
    m_sarrinIn("shortarrayin", m_sarrin),
    m_farrinIn("floatarrayin", m_farrin),
    m_darrinIn("doublearrayin", m_darrin),
    m_barrinIn("booleanarrayin", m_barrin),

    m_longout0Out("longout0", m_longout0),
    m_longout1Out("longout1", m_longout1),
    m_longout2Out("longout2", m_longout2),
    m_longout3Out("longout3", m_longout3),
    m_longout4Out("longout4", m_longout4),
    m_longout5Out("longout5", m_longout5),
    m_longout6Out("longout6", m_longout6),
    m_longout7Out("longout7", m_longout7),
    m_longout8Out("longout8", m_longout8),
    m_longout9Out("longout9", m_longout9),
    m_shortout0Out("shortout0", m_shortout0),
    m_shortout1Out("shortout1", m_shortout1),
    m_shortout2Out("shortout2", m_shortout2),
    m_shortout3Out("shortout3", m_shortout3),
    m_shortout4Out("shortout4", m_shortout4),
    m_shortout5Out("shortout5", m_shortout5),
    m_shortout6Out("shortout6", m_shortout6),
    m_shortout7Out("shortout7", m_shortout7),
    m_shortout8Out("shortout8", m_shortout8),
    m_shortout9Out("shortout9", m_shortout9),
    m_floatout0Out("floatout0", m_floatout0),
    m_floatout1Out("floatout1", m_floatout1),
    m_floatout2Out("floatout2", m_floatout2),
    m_floatout3Out("floatout3", m_floatout3),
    m_floatout4Out("floatout4", m_floatout4),
    m_floatout5Out("floatout5", m_floatout5),
    m_floatout6Out("floatout6", m_floatout6),
    m_floatout7Out("floatout7", m_floatout7),
    m_floatout8Out("floatout8", m_floatout8),
    m_floatout9Out("floatout9", m_floatout9),
    m_doubleout0Out("doubleout0", m_doubleout0),
    m_doubleout1Out("doubleout1", m_doubleout1),
    m_doubleout2Out("doubleout2", m_doubleout2),
    m_doubleout3Out("doubleout3", m_doubleout3),
    m_doubleout4Out("doubleout4", m_doubleout4),
    m_doubleout5Out("doubleout5", m_doubleout5),
    m_doubleout6Out("doubleout6", m_doubleout6),
    m_doubleout7Out("doubleout7", m_doubleout7),
    m_doubleout8Out("doubleout8", m_doubleout8),
    m_doubleout9Out("doubleout9", m_doubleout9),
    m_boolout0Out("boolout0", m_boolout0),
    m_boolout1Out("boolout1", m_boolout1),
    m_boolout2Out("boolout2", m_boolout2),
    m_boolout3Out("boolout3", m_boolout3),
    m_boolout4Out("boolout4", m_boolout4),
    m_boolout5Out("boolout5", m_boolout5),
    m_boolout6Out("boolout6", m_boolout6),
    m_boolout7Out("boolout7", m_boolout7),
    m_boolout8Out("boolout8", m_boolout8),
    m_boolout9Out("boolout9", m_boolout9),

    m_larrout0Out("longarrayout0", m_larrout0),
    m_larrout1Out("longarrayout1", m_larrout1),
    m_larrout2Out("longarrayout2", m_larrout2),
    m_larrout3Out("longarrayout3", m_larrout3),
    m_larrout4Out("longarrayout4", m_larrout4),
    m_larrout5Out("longarrayout5", m_larrout5),
    m_larrout6Out("longarrayout6", m_larrout6),
    m_larrout7Out("longarrayout7", m_larrout7),
    m_larrout8Out("longarrayout8", m_larrout8),
    m_larrout9Out("longarrayout9", m_larrout9),
    m_sarrout0Out("shortarrayout0", m_sarrout0),
    m_sarrout1Out("shortarrayout1", m_sarrout1),
    m_sarrout2Out("shortarrayout2", m_sarrout2),
    m_sarrout3Out("shortarrayout3", m_sarrout3),
    m_sarrout4Out("shortarrayout4", m_sarrout4),
    m_sarrout5Out("shortarrayout5", m_sarrout5),
    m_sarrout6Out("shortarrayout6", m_sarrout6),
    m_sarrout7Out("shortarrayout7", m_sarrout7),
    m_sarrout8Out("shortarrayout8", m_sarrout8),
    m_sarrout9Out("shortarrayout9", m_sarrout9),
    m_farrout0Out("floatarrayout0", m_farrout0),
    m_farrout1Out("floatarrayout1", m_farrout1),
    m_farrout2Out("floatarrayout2", m_farrout2),
    m_farrout3Out("floatarrayout3", m_farrout3),
    m_farrout4Out("floatarrayout4", m_farrout4),
    m_farrout5Out("floatarrayout5", m_farrout5),
    m_farrout6Out("floatarrayout6", m_farrout6),
    m_farrout7Out("floatarrayout7", m_farrout7),
    m_farrout8Out("floatarrayout8", m_farrout8),
    m_farrout9Out("floatarrayout9", m_farrout9),
    m_darrout0Out("doublearrayout0", m_darrout0),
    m_darrout1Out("doublearrayout1", m_darrout1),
    m_darrout2Out("doublearrayout2", m_darrout2),
    m_darrout3Out("doublearrayout3", m_darrout3),
    m_darrout4Out("doublearrayout4", m_darrout4),
    m_darrout5Out("doublearrayout5", m_darrout5),
    m_darrout6Out("doublearrayout6", m_darrout6),
    m_darrout7Out("doublearrayout7", m_darrout7),
    m_darrout8Out("doublearrayout8", m_darrout8),
    m_darrout9Out("doublearrayout9", m_darrout9),
    m_barrout0Out("booleanarrayout0", m_barrout0),
    m_barrout1Out("booleanarrayout1", m_barrout1),
    m_barrout2Out("booleanarrayout2", m_barrout2),
    m_barrout3Out("booleanarrayout3", m_barrout3),
    m_barrout4Out("booleanarrayout4", m_barrout4),
    m_barrout5Out("booleanarrayout5", m_barrout5),
    m_barrout6Out("booleanarrayout6", m_barrout6),
    m_barrout7Out("booleanarrayout7", m_barrout7),
    m_barrout8Out("booleanarrayout8", m_barrout8),
    m_barrout9Out("booleanarrayout9", m_barrout9),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("datain", m_datainIn);
  
  // Set OutPort buffer
  //registerOutPort("dataout", m_dataoutOut);
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("duplicator", "duplicator", m_duplicator);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

Duplicator::~Duplicator()
{
}


RTC::ReturnCode_t Duplicator::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("OutPort_Num", m_OutPort_Num, "2");
  
  // </rtc-template>
  std::cout << "Duplicator Start." << std::endl;  
  return RTC::RTC_OK;
}




RTC::ReturnCode_t Duplicator::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}

bool svc_flag=false;
int svc_type=0;
int svc_num=0;
bool activate_flag=true;

RTC::ReturnCode_t Duplicator::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){

  if(svc_flag)
    type = svc_type;
  else{
    if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double" && m_DataType!="bool" && m_DataType!="shortArray" && m_DataType!="longArray" && m_DataType!="floatArray" && m_DataType!="doubleArray" && m_DataType!="boolArray"){
      std::cout << "DataType ERROR" << std::endl;
      return RTC::RTC_ERROR;
    }
    if(m_DataType=="short")
      type = SHORT;
    else if(m_DataType=="long")
      type = LONG;
    else if(m_DataType=="float")
      type = FLOAT;
    else if(m_DataType=="double")
      type = DOUBLE;
    else if(m_DataType=="bool")
      type = BOOL;
    else if(m_DataType=="shortArray")
      type = SHORTARRAY;
    else if(m_DataType=="longArray")
      type = LONGARRAY;
    else if(m_DataType=="floatArray")
      type = FLOATARRAY;
    else if(m_DataType=="doubleArray")
      type = DOUBLEARRAY;
    else if(m_DataType=="boolArray")
      type = BOOLARRAY;
  }

  if(svc_flag)
    num = svc_num;
  else
    num = m_OutPort_Num;

  if(1 > num || num > 10){
    std::cout << "OutPort_Num ERROR" << std::endl;
    return RTC::RTC_ERROR;
  }  


  switch(type){
    case SHORT:
      registerInPort("shortin", m_shortinIn);
      if(num > 0)
        registerOutPort("shortout0", m_shortout0Out);
      if(num > 1)
        registerOutPort("shortout1", m_shortout1Out);
      if(num > 2)
        registerOutPort("shortout2", m_shortout2Out);
      if(num > 3)
        registerOutPort("shortout3", m_shortout3Out);
      if(num > 4)
        registerOutPort("shortout4", m_shortout4Out);
      if(num > 5)
        registerOutPort("shortout5", m_shortout5Out);
      if(num > 6)
        registerOutPort("shortout6", m_shortout6Out);
      if(num > 7)
        registerOutPort("shortout7", m_shortout7Out);
      if(num > 8)
        registerOutPort("shortout8", m_shortout8Out);
      if(num > 9)
        registerOutPort("shortout9", m_shortout9Out);
    break;

    case LONG:
      registerInPort("longin", m_longinIn);
      if(num > 0)
        registerOutPort("longout0", m_longout0Out);
      if(num > 1)
        registerOutPort("longout1", m_longout1Out);
      if(num > 2)
        registerOutPort("longout2", m_longout2Out);
      if(num > 3)
        registerOutPort("longout3", m_longout3Out);
      if(num > 4)
        registerOutPort("longout4", m_longout4Out);
      if(num > 5)
        registerOutPort("longout5", m_longout5Out);
      if(num > 6)
        registerOutPort("longout6", m_longout6Out);
      if(num > 7)
        registerOutPort("longout7", m_longout7Out);
      if(num > 8)
        registerOutPort("longout8", m_longout8Out);
      if(num > 9)
        registerOutPort("longout9", m_longout9Out);
    break;

    case FLOAT:
      registerInPort("floatin", m_floatinIn);
      if(num > 0)
        registerOutPort("floatout0", m_floatout0Out);
      if(num > 1)
        registerOutPort("floatout1", m_floatout1Out);
      if(num > 2)
        registerOutPort("floatout2", m_floatout2Out);
      if(num > 3)
        registerOutPort("floatout3", m_floatout3Out);
      if(num > 4)
        registerOutPort("floatout4", m_floatout4Out);
      if(num > 5)
        registerOutPort("floatout5", m_floatout5Out);
      if(num > 6)
        registerOutPort("floatout6", m_floatout6Out);
      if(num > 7)
        registerOutPort("floatout7", m_floatout7Out);
      if(num > 8)
        registerOutPort("floatout8", m_floatout8Out);
      if(num > 9)
        registerOutPort("floatout9", m_floatout9Out);
    break;

    case DOUBLE:
      registerInPort("doublein", m_doubleinIn);
      if(num > 0)
        registerOutPort("doubleout0", m_doubleout0Out);
      if(num > 1)
        registerOutPort("doubleout1", m_doubleout1Out);
      if(num > 2)
        registerOutPort("doubleout2", m_doubleout2Out);
      if(num > 3)
        registerOutPort("doubleout3", m_doubleout3Out);
      if(num > 4)
        registerOutPort("doubleout4", m_doubleout4Out);
      if(num > 5)
        registerOutPort("doubleout5", m_doubleout5Out);
      if(num > 6)
        registerOutPort("doubleout6", m_doubleout6Out);
      if(num > 7)
        registerOutPort("doubleout7", m_doubleout7Out);
      if(num > 8)
        registerOutPort("doubleout8", m_doubleout8Out);
      if(num > 9)
        registerOutPort("doubleout9", m_doubleout9Out);
    break;

    case BOOL:
      registerInPort("booleanin", m_boolinIn);
      if(num > 0)
        registerOutPort("booleanout0", m_boolout0Out);
      if(num > 1)
        registerOutPort("booleanout1", m_boolout1Out);
      if(num > 2)
        registerOutPort("booleanout2", m_boolout2Out);
      if(num > 3)
        registerOutPort("booleanout3", m_boolout3Out);
      if(num > 4)
        registerOutPort("booleanout4", m_boolout4Out);
      if(num > 5)
        registerOutPort("booleanout5", m_boolout5Out);
      if(num > 6)
        registerOutPort("booleanout6", m_boolout6Out);
      if(num > 7)
        registerOutPort("booleanout7", m_boolout7Out);
      if(num > 8)
        registerOutPort("booleanout8", m_boolout8Out);
      if(num > 9)
        registerOutPort("booleanout9", m_boolout9Out);
    break;

    case SHORTARRAY:
      registerInPort("shortArrayin", m_sarrinIn);
      if(num > 0)
        registerOutPort("shortArrayout0", m_sarrout0Out);
      if(num > 1)
        registerOutPort("shortArrayout1", m_sarrout1Out);
      if(num > 2)
        registerOutPort("shortArrayout2", m_sarrout2Out);
      if(num > 3)
        registerOutPort("shortArrayout3", m_sarrout3Out);
      if(num > 4)
        registerOutPort("shortArrayout4", m_sarrout4Out);
      if(num > 5)
        registerOutPort("shortArrayout5", m_sarrout5Out);
      if(num > 6)
        registerOutPort("shortArrayout6", m_sarrout6Out);
      if(num > 7)
        registerOutPort("shortArrayout7", m_sarrout7Out);
      if(num > 8)
        registerOutPort("shortArrayout8", m_sarrout8Out);
      if(num > 9)
        registerOutPort("shortArrayout9", m_sarrout9Out);
    break;

    case LONGARRAY:
      registerInPort("longArrayin", m_larrinIn);
      if(num > 0)
        registerOutPort("longArrayout0", m_larrout0Out);
      if(num > 1)
        registerOutPort("longArrayout1", m_larrout1Out);
      if(num > 2)
        registerOutPort("longArrayout2", m_larrout2Out);
      if(num > 3)
        registerOutPort("longArrayout3", m_larrout3Out);
      if(num > 4)
        registerOutPort("longArrayout4", m_larrout4Out);
      if(num > 5)
        registerOutPort("longArrayout5", m_larrout5Out);
      if(num > 6)
        registerOutPort("longArrayout6", m_larrout6Out);
      if(num > 7)
        registerOutPort("longArrayout7", m_larrout7Out);
      if(num > 8)
        registerOutPort("longArrayout8", m_larrout8Out);
      if(num > 9)
        registerOutPort("longArrayout9", m_larrout9Out);
    break;

    case FLOATARRAY:
      registerInPort("floatArrayin", m_farrinIn);
      if(num > 0)
        registerOutPort("floatArrayout0", m_farrout0Out);
      if(num > 1)
        registerOutPort("floatArrayout1", m_farrout1Out);
      if(num > 2)
        registerOutPort("floatArrayout2", m_farrout2Out);
      if(num > 3)
        registerOutPort("floatArrayout3", m_farrout3Out);
      if(num > 4)
        registerOutPort("floatArrayout4", m_farrout4Out);
      if(num > 5)
        registerOutPort("floatArrayout5", m_farrout5Out);
      if(num > 6)
        registerOutPort("floatArrayout6", m_farrout6Out);
      if(num > 7)
        registerOutPort("floatArrayout7", m_farrout7Out);
      if(num > 8)
        registerOutPort("floatArrayout8", m_farrout8Out);
      if(num > 9)
        registerOutPort("floatArrayout9", m_farrout9Out);
    break;

    case DOUBLEARRAY:
      registerInPort("doubleArrayin", m_darrinIn);
      if(num > 0)
        registerOutPort("doubleArrayout0", m_darrout0Out);
      if(num > 1)
        registerOutPort("doubleArrayout1", m_darrout1Out);
      if(num > 2)
        registerOutPort("doubleArrayout2", m_darrout2Out);
      if(num > 3)
        registerOutPort("doubleArrayout3", m_darrout3Out);
      if(num > 4)
        registerOutPort("doubleArrayout4", m_darrout4Out);
      if(num > 5)
        registerOutPort("doubleArrayout5", m_darrout5Out);
      if(num > 6)
        registerOutPort("doubleArrayout6", m_darrout6Out);
      if(num > 7)
        registerOutPort("doubleArrayout7", m_darrout7Out);
      if(num > 8)
        registerOutPort("doubleArrayout8", m_darrout8Out);
      if(num > 9)
        registerOutPort("doubleArrayout9", m_darrout9Out);
    break;

    case BOOLARRAY:
      registerInPort("booleanArrayin", m_barrinIn);
      if(num > 0)
        registerOutPort("booleanArrayout0", m_barrout0Out);
      if(num > 1)
        registerOutPort("booleanArrayout1", m_barrout1Out);
      if(num > 2)
        registerOutPort("booleanArrayout2", m_barrout2Out);
      if(num > 3)
        registerOutPort("booleanArrayout3", m_barrout3Out);
      if(num > 4)
        registerOutPort("booleanArrayout4", m_barrout4Out);
      if(num > 5)
        registerOutPort("booleanArrayout5", m_barrout5Out);
      if(num > 6)
        registerOutPort("booleanArrayout6", m_barrout6Out);
      if(num > 7)
        registerOutPort("booleanArrayout7", m_barrout7Out);
      if(num > 8)
        registerOutPort("booleanArrayout8", m_barrout8Out);
      if(num > 9)
        registerOutPort("booleanArrayout9", m_barrout9Out);
    break;
  }

  activate_flag=false;
  }

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Duplicator::onDeactivated(RTC::UniqueId ec_id)
{
  m_shortinIn.read();
  m_longinIn.read();
  m_floatinIn.read();
  m_doubleinIn.read();
  m_boolinIn.read();
  m_sarrinIn.read();
  m_larrinIn.read();
  m_farrinIn.read();
  m_darrinIn.read();
  m_barrinIn.read();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Duplicator::onExecute(RTC::UniqueId ec_id)
{
  bool flag=false;
  static short shortnum;
  static long longnum;
  static float floatnum;
  static double doublenum;
  static bool boolnum;
  int selement = 0, lelement = 0, felement = 0, delement = 0, belement = 0;
  std::vector<short> sarrnum;
  std::vector<long> larrnum;
  std::vector<float> farrnum;
  std::vector<double> darrnum;
  std::vector<bool> barrnum;

  if(m_shortinIn.isNew()){
    m_shortinIn.read();
    shortnum = m_shortin.data;
    flag = true;
  }
  if(m_longinIn.isNew()){
    m_longinIn.read();
    longnum = m_longin.data;
    flag = true;
  }
  if(m_floatinIn.isNew()){
    m_floatinIn.read();
    floatnum = m_floatin.data;
    flag = true;
  }
  if(m_doubleinIn.isNew()){
    m_doubleinIn.read();
    doublenum = m_doublein.data;
    flag = true;
  }
  if(m_boolinIn.isNew()){
    m_boolinIn.read();
    boolnum = m_boolin.data;
    flag = true;
  }
  if(m_sarrinIn.isNew()){
    m_sarrinIn.read();
    selement = m_sarrin.data.length();
    for(int i=0; i<selement; i++)
      sarrnum.push_back(m_sarrin.data[i]);
    flag = true;
  }
  if(m_larrinIn.isNew()){
    m_larrinIn.read();
    lelement = m_larrin.data.length();
    for(int i=0; i<lelement; i++)
      larrnum.push_back(m_larrin.data[i]);
    flag = true;
  }
  if(m_farrinIn.isNew()){
    m_farrinIn.read();
    felement = m_farrin.data.length();
    for(int i=0; i<felement; i++)
      farrnum.push_back(m_farrin.data[i]);
    flag = true;
  }
  if(m_darrinIn.isNew()){
    m_darrinIn.read();
    delement = m_darrin.data.length();
    for(int i=0; i<delement; i++)
      darrnum.push_back(m_darrin.data[i]);
    flag = true;
  }
  if(m_barrinIn.isNew()){
    m_barrinIn.read();
    belement = m_barrin.data.length();
    for(int i=0; i<belement; i++)
      barrnum.push_back(m_barrin.data[i]);
    flag = true;
  }

  if(flag){
    if(num > 0){
      m_shortout0.data = shortnum;
      m_shortout0Out.write();
      m_longout0.data = longnum;
      m_longout0Out.write();
      m_floatout0.data = floatnum;
      m_floatout0Out.write();
      m_doubleout0.data = doublenum;
      m_doubleout0Out.write();
      m_boolout0.data = boolnum;
      m_boolout0Out.write();
      m_barrout0.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout0.data[i] = barrnum[i];
      m_barrout0Out.write();
      m_sarrout0.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout0.data[i] = sarrnum[i];
      m_sarrout0Out.write();

      m_larrout0.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout0.data[i] = larrnum[i];
      m_larrout0Out.write();
      m_farrout0.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout0.data[i] = farrnum[i];
      m_farrout0Out.write();
      m_darrout0.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout0.data[i] = darrnum[i];
      m_darrout0Out.write();
    }
    if(num > 1){
      m_shortout1.data = shortnum;
      m_shortout1Out.write();
      m_longout1.data = longnum;
      m_longout1Out.write();
      m_floatout1.data = floatnum;
      m_floatout1Out.write();
      m_doubleout1.data = doublenum;
      m_doubleout1Out.write();
      m_boolout1.data = boolnum;
      m_boolout1Out.write();
      m_barrout1.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout1.data[i] = barrnum[i];
      m_barrout1Out.write();
      m_sarrout1.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout1.data[i] = sarrnum[i];
      m_sarrout1Out.write();
      m_larrout1.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout1.data[i] = larrnum[i];
      m_larrout1Out.write();
      m_farrout1.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout1.data[i] = farrnum[i];
      m_farrout1Out.write();
      m_darrout1.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout1.data[i] = darrnum[i];
      m_darrout1Out.write();
    }
    if(num > 2){
      m_shortout2.data = shortnum;
      m_shortout2Out.write();
      m_longout2.data = longnum;
      m_longout2Out.write();
      m_floatout2.data = floatnum;
      m_floatout2Out.write();
      m_doubleout2.data = doublenum;
      m_doubleout2Out.write();
      m_boolout2.data = boolnum;
      m_boolout2Out.write();
      m_barrout2.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout2.data[i] = barrnum[i];
      m_barrout2Out.write();
      m_sarrout2.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout2.data[i] = sarrnum[i];
      m_sarrout2Out.write();
      m_larrout2.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout2.data[i] = larrnum[i];
      m_larrout2Out.write();
      m_farrout2.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout2.data[i] = farrnum[i];
      m_farrout2Out.write();
      m_darrout2.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout2.data[i] = darrnum[i];
      m_darrout2Out.write();
    }
    if(num > 3){
      m_shortout3.data = shortnum;
      m_shortout3Out.write();
      m_longout3.data = longnum;
      m_longout3Out.write();
      m_floatout3.data = floatnum;
      m_floatout3Out.write();
      m_doubleout3.data = doublenum;
      m_doubleout3Out.write();
      m_boolout3.data = boolnum;
      m_boolout3Out.write();
      m_barrout3.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout3.data[i] = barrnum[i];
      m_barrout3Out.write();
      m_sarrout3.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout3.data[i] = sarrnum[i];
      m_sarrout3Out.write();
      m_larrout3.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout3.data[i] = larrnum[i];
      m_larrout3Out.write();
      m_farrout3.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout3.data[i] = farrnum[i];
      m_farrout3Out.write();
      m_darrout3.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout3.data[i] = darrnum[i];
      m_darrout3Out.write();
    }
    if(num > 4){
      m_shortout4.data = shortnum;
      m_shortout4Out.write();
      m_longout4.data = longnum;
      m_longout4Out.write();
      m_floatout4.data = floatnum;
      m_floatout4Out.write();
      m_doubleout4.data = doublenum;
      m_doubleout4Out.write();
      m_boolout4.data = boolnum;
      m_boolout4Out.write();
      m_barrout4.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout4.data[i] = barrnum[i];
      m_barrout4Out.write();
      m_sarrout4.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout4.data[i] = sarrnum[i];
      m_sarrout4Out.write();
      m_larrout4.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout4.data[i] = larrnum[i];
      m_larrout4Out.write();
      m_farrout4.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout4.data[i] = farrnum[i];
      m_farrout4Out.write();
      m_darrout4.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout4.data[i] = darrnum[i];
      m_darrout4Out.write();
    }
    if(num > 5){
      m_shortout5.data = shortnum;
      m_shortout5Out.write();
      m_longout5.data = longnum;
      m_longout5Out.write();
      m_floatout5.data = floatnum;
      m_floatout5Out.write();
      m_doubleout5.data = doublenum;
      m_doubleout5Out.write();
      m_boolout5.data = boolnum;
      m_boolout5Out.write();
      m_barrout5.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout5.data[i] = barrnum[i];
      m_barrout5Out.write();
      m_sarrout5.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout5.data[i] = sarrnum[i];
      m_sarrout5Out.write();
      m_larrout5.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout5.data[i] = larrnum[i];
      m_larrout5Out.write();
      m_farrout5.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout5.data[i] = farrnum[i];
      m_farrout5Out.write();
      m_darrout5.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout5.data[i] = darrnum[i];
      m_darrout5Out.write();
    }
    if(num > 6){
      m_shortout6.data = shortnum;
      m_shortout6Out.write();
      m_longout6.data = longnum;
      m_longout6Out.write();
      m_floatout6.data = floatnum;
      m_floatout6Out.write();
      m_doubleout6.data = doublenum;
      m_doubleout6Out.write();
      m_boolout6.data = boolnum;
      m_boolout6Out.write();
      m_barrout6.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout6.data[i] = barrnum[i];
      m_barrout6Out.write();
      m_sarrout6.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout6.data[i] = sarrnum[i];
      m_sarrout6Out.write();
      m_larrout6.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout6.data[i] = larrnum[i];
      m_larrout6Out.write();
      m_farrout6.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout6.data[i] = farrnum[i];
      m_farrout6Out.write();
      m_darrout6.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout6.data[i] = darrnum[i];
      m_darrout6Out.write();
    }
    if(num > 7){
      m_shortout7.data = shortnum;
      m_shortout7Out.write();
      m_longout7.data = longnum;
      m_longout7Out.write();
      m_floatout7.data = floatnum;
      m_floatout7Out.write();
      m_doubleout7.data = doublenum;
      m_doubleout7Out.write();
      m_boolout7.data = boolnum;
      m_boolout7Out.write();
      m_barrout7.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout7.data[i] = barrnum[i];
      m_barrout7Out.write();
      m_sarrout7.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout7.data[i] = sarrnum[i];
      m_sarrout7Out.write();
      m_larrout7.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout7.data[i] = larrnum[i];
      m_larrout7Out.write();
      m_farrout7.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout7.data[i] = farrnum[i];
      m_farrout7Out.write();
      m_darrout7.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout7.data[i] = darrnum[i];
      m_darrout7Out.write();
    }
    if(num > 8){
      m_shortout8.data = shortnum;
      m_shortout8Out.write();
      m_longout8.data = longnum;
      m_longout8Out.write();
      m_floatout8.data = floatnum;
      m_floatout8Out.write();
      m_doubleout8.data = doublenum;
      m_doubleout8Out.write();
      m_boolout8.data = boolnum;
      m_boolout8Out.write();
      m_barrout8.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout8.data[i] = barrnum[i];
      m_barrout8Out.write();
      m_sarrout8.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout8.data[i] = sarrnum[i];
      m_sarrout8Out.write();
      m_larrout8.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout8.data[i] = larrnum[i];
      m_larrout8Out.write();
      m_farrout8.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout8.data[i] = farrnum[i];
      m_farrout8Out.write();
      m_darrout8.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout8.data[i] = darrnum[i];
      m_darrout8Out.write();
    }
    if(num > 9){
      m_shortout9.data = shortnum;
      m_shortout9Out.write();
      m_longout9.data = longnum;
      m_longout9Out.write();
      m_floatout9.data = floatnum;
      m_floatout9Out.write();
      m_doubleout9.data = doublenum;
      m_doubleout9Out.write();
      m_boolout9.data = boolnum;
      m_boolout9Out.write();
      m_barrout9.data.length(belement);
      for(int i=0, len=barrnum.size(); i<len; i++)
        m_barrout9.data[i] = barrnum[i];
      m_barrout9Out.write();
      m_sarrout9.data.length(selement);
      for(int i=0, len=sarrnum.size(); i<len; i++)
        m_sarrout9.data[i] = sarrnum[i];
      m_sarrout9Out.write();
      m_larrout9.data.length(lelement);
      for(int i=0, len=larrnum.size(); i<len; i++)
        m_larrout9.data[i] = larrnum[i];
      m_larrout9Out.write();
      m_farrout9.data.length(felement);
      for(int i=0, len=farrnum.size(); i<len; i++)
        m_farrout9.data[i] = farrnum[i];
      m_farrout9Out.write();
      m_darrout9.data.length(delement);
      for(int i=0, len=darrnum.size(); i<len; i++)
        m_darrout9.data[i] = darrnum[i];
      m_darrout9Out.write();
    }
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Duplicator::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Duplicator::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Duplicator::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DuplicatorInit(RTC::Manager* manager)
  {
    RTC::Properties profile(duplicator_spec);
    manager->registerFactory(profile,
                             RTC::Create<Duplicator>,
                             RTC::Delete<Duplicator>);
  }
  
};


