// -*- C++ -*-
/*!
 * @file  Gainer.h
 * @brief Gainer component
 * @date  $Date$
 *
 * $Id$
 */

#ifndef GAINER_H
#define GAINER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "GainerSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

class Gainer
  : public RTC::DataFlowComponentBase
{
 public:
  Gainer(RTC::Manager* manager);
  ~Gainer();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_DataType;
  double m_Gain;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedShort m_shortin;
  InPort<TimedShort> m_shortinIn;
  TimedLong m_longin;
  InPort<TimedLong> m_longinIn;
  TimedFloat m_floatin;
  InPort<TimedFloat> m_floatinIn;
  TimedDouble m_doublein;
  InPort<TimedDouble> m_doubleinIn;

  TimedShortSeq m_sarrin;
  InPort<TimedShortSeq> m_sarrinIn;
  TimedLongSeq m_larrin;
  InPort<TimedLongSeq> m_larrinIn;
  TimedFloatSeq m_farrin;
  InPort<TimedFloatSeq> m_farrinIn;
  TimedDoubleSeq m_darrin;
  InPort<TimedDoubleSeq> m_darrinIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedShort m_shortout;
  OutPort<TimedShort> m_shortoutOut;
  TimedLong m_longout;
  OutPort<TimedLong> m_longoutOut;
  TimedFloat m_floatout;
  OutPort<TimedFloat> m_floatoutOut;
  TimedDouble m_doubleout;
  OutPort<TimedDouble> m_doubleoutOut;

  TimedShortSeq m_sarrout;
  OutPort<TimedShortSeq> m_sarroutOut;
  TimedLongSeq m_larrout;
  OutPort<TimedLongSeq> m_larroutOut;
  TimedFloatSeq m_farrout;
  OutPort<TimedFloatSeq> m_farroutOut;
  TimedDoubleSeq m_darrout;
  OutPort<TimedDoubleSeq> m_darroutOut;

  int type;
  double gain;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_PrvPortPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  gainerSVC_impl m_gainer;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;

};


extern "C"
{
  void GainerInit(RTC::Manager* manager);
};

#endif // GAINER_H
